// **********************************************************************************
//                               STATBLOCKPASTER.JS                                 *
//                                Code: Corey Reid                                  *
//                                                                                  *
//  This widget creates D&D "stat blocks" for monsters. I owe a big thanks to       *
//  dand on the Konfabulator forum for pointing me to the xml parsing used by       *
//  Arlo Rose's miniWeather widget. And another to Ricky Romano for so patiently    *
//  walking me through the cross-platform issues that I knew nothing about.         *
//                                                                                  *
//  Thanks to dand, Arlo and Ricky.                                                 *
//                                                                                  *
//                               (c) 2004 Corey Reid                                *
//                                                                                  *
// *************************** PRE-FUNCTION LOAD ************************************
//                                                                                  *
// We need to load the xmldom.js file so that our functions know what to do.        *
//                                                                                  *
// **********************************************************************************

eval(runCommand("cat xmldom.js"));

// ************************** DISPLAY FUNCTIONS *************************************
//                                                                                  *
// These functions pop up forms and handle user input.                              *
//                                                                                  *
// **********************************************************************************
//                                                                                  *
// function displayTypeSelectForm()                                                 *
//                                                                                  *
// This function generates the initial form to allow users to pick creature types.  *
// It first generates an array of type names using the files found in the Data      *
// folder, and sends the chosen type to the function displayMonsterSelectForm.      *
//                                                                                  *
// **********************************************************************************

function displayTypeSelectForm() {
    var formfields = new Array();
    formfields[0] = new FormField();
    formfields[0].name = 'typePicker';
    formfields[0].type = 'popup';
    formfields[0].title = 'Pick A Type';
    formfields[0].option = getFileNames();
    formfields[0].description = 'Choose a creature type.';

    FormResults = form(formfields, 'Type Picker', 'Choose');
    
    if (FormResults != null) { displayMonsterSelectForm (FormResults); }
}

// **********************************************************************************
//                                                                                  *
// function displayMonsterSelectForm(type)                                          *
//                                                                                  *
// This function generates the second form to allow users to pick creatures of the  *
// desired type. It first reads the selected type file and creates a list of names, *
// and then passes the selected name to pasteStatBlock.                             *
//                                                                                  *
// **********************************************************************************

function displayMonsterSelectForm (type) {
    var xmlData = getTypeXML(type);
    var monsterList = getMonsterList(xmlData);
    var nameList = getNames(monsterList);

    var formfields = new Array();
    formfields[0] = new FormField();
    formfields[0].name = 'monsterPicker';
    formfields[0].type = 'popup';
    formfields[0].title = 'Now Pick A Monster';
    formfields[0].option = nameList;
    formfields[0].description = 'Choose a monster to copy.';
    
    FormResults = form(formfields, 'Monster Picker', 'Copy');
    
    if (FormResults != null) { pasteStatBlock (FormResults, monsterList); }

}

// **********************************************************************************
//                                                                                  *
// function displayCopyStatBlockForm()                                              *
//                                                                                  *
// This function displays a form that allows users to enter info on stat blocks to  *
// be copied into the XML data. It passes the defined name and type and the copied  *
// text for the new monster's stat block to copyStatBlock.                          *
//                                                                                  *
// **********************************************************************************

function displayCopyStatBlockForm(selectedText) {
    var confirmationMessage = "The statblock you selected is:\n\n" +
        selectedText.substring(0, 1000) + "\n\n";
    
    var formfields = new Array();
    formfields[0] = new FormField();
    formfields[0].name = 'monsterName';
    formfields[0].type = 'text';
    formfields[0].title = 'Name';
    formfields[0].description = confirmationMessage;
    formfields[1] = new FormField();
    formfields[1].name = 'typePicker';
    formfields[1].type = 'popup';
    formfields[1].title = 'Pick A Type';
    formfields[1].option = getFileNames();
    formfields[1].defaultValue = getFileNames()[3];
    formfields[1].description = 'What type of Creature is this?';

    FormResults = form(formfields, "Copy New Stat Block", "Copy");

    if (FormResults != null ) { copyStatBlock(FormResults, selectedText); }
}

// **********************************************************************************
//                      XML FUNCTIONS                                               *
//                                                                                  *
// These functions handle the XML I/O and so on. They rely on xmldom.js, which is   *
// way more complicated than I can understand.                                      *
//                                                                                  *
// * getTypeXML reads the XML data and returns the DOM object                       *
//                                                                                  *
// * getMonsterList turns the DOM object into a set of Node objects                 *
//                                                                                  *
// * getNames returns an array of strings from the <name> element items in the Node *
// objects supplied.                                                                *
//                                                                                  *
// * saveXML takes a DOM object, adds the selected text as a new monster, and saves *
// the DOM object to a file                                                         *
//                                                                                  *
// * pasteStatBlock searches through the Node objects for one whose <name> matches  *
// the selected name and copies the <stat_block> text to the clipboard.             *
//                                                                                  *
// * copyStatBlock passes the copied text off to displayCopyStatBlockForm, verifies *
// the provided name and passes the copied text over to saveXML.                    *
//                                                                                  *
// * xmlError just displays an alert if xmldom.js passes an error back from any     *
// function.                                                                        *
//                                                                                  *
// **********************************************************************************

function getTypeXML (type) {
    var filename = "Data/" + type + ".xml";
    var xml = runCommand("cat \"" + filename + "\"");
    if (xml != null) { var objDom = new XMLDoc(xml, xmlError); }
    else { xmlError("No XML file could be found. Bummer."); }
    return objDom;
}

function getMonsterList(xmlData) {
    var xmlDocNode = xmlData.docNode;
    return xmlDocNode.getElements("monster");
}


function getNames(monsterList) {
    var nameList = new Array();
    for ( i = 0; i < monsterList.length; i++ ) {
        var monster = monsterList[i];
        var monsterName = monster.getElements("name");
        nameList[i] = monsterName[0].getText();
    }
    return nameList;
}

function saveXML (xmlData, selectedText) {
    var typeDataTree = xmlData.docNode;
    var newStatBlock = xmlData.createXMLNode( "<monster>\n  <name>" + 
        FormResults[0] + "</name>" + "\n  <stat_block>" + selectedText + 
        "</stat_block>\n </monster>");
    xmlData = xmlData.insertNodeInto(typeDataTree, newStatBlock);
    var xmlOutput = xmlData.getUnderlyingXMLText();
    filename = "\"Data/" + FormResults[1] + ".xml\"";
    runCommand("echo \"" + xmlOutput + "\" > " + filename);
}

function pasteStatBlock (selectedName, monsterList) {
    for ( i = 0; i < monsterList.length; i++ ) {
        var monster = monsterList[i];
        var monsterName = monster.getElements("name");

        if ( monsterName[0].getText() == selectedName ) {
            var monsterStatBlock = monster.getElements("stat_block");
            var monsterStatBlockText = monsterStatBlock[0].getText();
            system.clipboard = monsterStatBlockText;
        }
    }
}

function copyStatBlock(newMonsterNameAndType, statBlockText) { 
   if (newMonsterNameAndType != null ) {
        var typeData = getTypeXML(newMonsterNameAndType[1]);
        var nameList = getNames(getMonsterList(typeData));
        var saveMonster = 0;
        for (i in nameList) { 
            if (newMonsterNameAndType[0] == nameList[i]) {
                saveMonster = 1;
                var nameConflict = alert
                    ("There's already a beastie with that name. Try another.",
                        "Okay", "Forget It");
                if (nameConflict == 2 ) { 
                    break; 
                }
                if (nameConflict == 1 ) { 
                    displayCopyStatBlockForm(statBlockText);
                }
               
            }
        }
        if (saveMonster == 0) { saveXML (typeData, statBlockText); }
    }
}

function xmlError(e) { alert(e); }

// **********************************************************************************
//                            OTHER FUNCTIONS                                       *
// Not XML related, don't display dialogs -- these are "utility" functions          *
//                                                                                  *
// * setKeyLabel generates the label that appears on the widget, based on the       *
// current preference, and sets the hotkey values for both copy and paste           *
// functions. It has to use the little split-into-array-and-grab-just-the-first-two *
// elements trick because we have two versions of the required text -- one for Mac  *
// and one for Windows.                                                             *
//                                                                                  *
// * getFileNames rummages through the "Data" directory and returns an array of     *
// whatever it finds in there. This function uses filesystem.getDirectoryContents   *
// to avoid (we hope) cross-platform issues.                                        *
//                                                                                  *
// **********************************************************************************

function setKeyLabel (keyValue) {
    TextAsArray = keyText.data.split("+");
	keyText.data = (TextAsArray[0] + "+" + TextAsArray[1] + "+");
    keyText.data = keyText.data+keyValue;
    hkey1.key = keyValue;
    hkey2.key = keyValue;
}

function getFileNames() {
    var dataFilesArray = filesystem.getDirectoryContents("Data/", false);
    for ( var i in dataFilesArray ) {
//      splitDataEntry = dataFilesArray[i].split("/");
//      dataFilesArray[i] = splitDataEntry[1].substring
//		(0, (splitDataEntry[1].length-4));
		dataFilesArray[i] = dataFilesArray[i].substring
			(0, (dataFilesArray[i].length-4));
    }
    return dataFilesArray;
}

// ***********************  BEGIN  **************************************************
//                                                                                  *
// To start we just call setKeyLabel().                                             *
//                                                                                  *
// **********************************************************************************

setKeyLabel (preferences.keyPref.value);

// ********************** END *******************************************************